@@ -1,11 +1,14 @@
+"use strict";
+
 var halloPlugins = {
     'halloformat': {},
     'halloheadings': {formatBlocks: ["p", "h2", "h3", "h4", "h5"]},
     'hallolists': {},
     'hallohr': {},
     'halloreundo': {},
-    'hallowagtaillink': {},
+    'hallowagtaillink': {}
 };
+
 function registerHalloPlugin(name, opts) {
     halloPlugins[name] = (opts || {});
 }
@@ -30,7 +33,7 @@ function makeRichTextEditable(id) {
 
     richText.hallo({
         toolbar: 'halloToolbarFixed',
-        toolbarcssClass: 'testy',
+        toolbarCssClass: (input.closest('.object').hasClass('full')) ? 'full' : '',
         plugins: halloPlugins
     }).bind('hallomodified', function(event, data) {
         input.val(data.content);
@@ -53,57 +56,60 @@ function insertRichTextDeleteControl(elem) {
     });
 }
 
-function initDateChoosers(context) {
-    $('input.friendly_date', context).datepicker({
-        dateFormat: 'd M yy', constrainInput: false, /* showOn: 'button', */ firstDay: 1
-    });
-
-    if(window.overrideDateInputFormat && window.overrideDateInputFormat !='') {
-        $('input.localized_date', context).datepicker({
-            dateFormat: window.overrideDateInputFormat, constrainInput: false, /* showOn: 'button', */ firstDay: 1
+function initDateChooser(id) {
+    if (window.dateTimePickerTranslations) {
+        $('#' + id).datetimepicker({
+            timepicker: false,
+            scrollInput:false,
+            format: 'Y-m-d',
+            i18n: {
+                lang: window.dateTimePickerTranslations
+            },
+            lang: 'lang'
         });
     } else {
-        $('input.localized_date', context).datepicker({
-            constrainInput: false, /* showOn: 'button', */ firstDay: 1
+        $('#' + id).datetimepicker({
+            timepicker: false,
+            scrollInput:false,
+            format: 'Y-m-d',
         });
     }
+}
 
-}
-function initFriendlyDateChooser(id) {
-    $('#' + id).datepicker({
-        dateFormat: 'd M yy', constrainInput: false, /* showOn: 'button', */ firstDay: 1
-    });
-}
-function initLocalizedDateChooser(id) {
-    if(window.overrideDateInputFormat && window.overrideDateInputFormat !='') {
-        $('#' + id).datepicker({
-            dateFormat: window.overrideDateInputFormat, constrainInput: false, /* showOn: 'button', */ firstDay: 1
+function initTimeChooser(id) {
+    if (window.dateTimePickerTranslations) {
+        $('#' + id).datetimepicker({
+            datepicker: false,
+            scrollInput:false,
+            format: 'H:i',
+            i18n: {
+                lang: window.dateTimePickerTranslations
+            },
+            lang: 'lang'
         });
     } else {
-        $('#' + id).datepicker({
-            constrainInput: false, /* showOn: 'button', */ firstDay: 1
+        $('#' + id).datetimepicker({
+            datepicker: false,
+            format: 'H:i',
         });
     }
-
 }
 
-function initTimeChoosers(context) {
-    $('input.friendly_time', context).timepicker({
-        timeFormat: 'g.ia'
-    });
-    $('input.localized_time', context).timepicker({
-        timeFormat: 'H:i', maxTime: '23:59'
-    });
-}
-function initFriendlyTimeChooser(id) {
-    $('#' + id).timepicker({
-        timeFormat: 'g.ia'
-    });
-}
-function initLocalizedTimeChooser(id) {
-    $('#' + id).timepicker({
-        timeFormat: 'H:i', maxTime: '23:59'
-    });
+function initDateTimeChooser(id) {
+    if (window.dateTimePickerTranslations) {
+        $('#' + id).datetimepicker({
+            format: 'Y-m-d H:i',
+            scrollInput:false,
+            i18n: {
+                lang: window.dateTimePickerTranslations
+            },
+            language: 'lang'
+        });
+    } else {
+    $('#' + id).datetimepicker({
+            format: 'Y-m-d H:i',
+        });
+    }
 }
 
 function initTagField(id, autocompleteUrl) {
@@ -198,7 +204,7 @@ function InlinePanel(opts) {
 
     self.updateMoveButtonDisabledStates = function() {
         if (opts.canOrder) {
-            forms = self.formsUl.children('li:visible');
+            var forms = self.formsUl.children('li:visible');
             forms.each(function(i) {
                 $('ul.controls .inline-child-move-up', this).toggleClass('disabled', i === 0).toggleClass('enabled', i !== 0);
                 $('ul.controls .inline-child-move-down', this).toggleClass('disabled', i === forms.length - 1).toggleClass('enabled', i != forms.length - 1);
@@ -241,7 +247,10 @@ function InlinePanel(opts) {
             }
             self.initChildControls(fixPrefix(opts.emptyChildFormPrefix));
             if (opts.canOrder) {
-                $(fixPrefix('#id_' + opts.emptyChildFormPrefix + '-ORDER')).val(formCount);
+                /* NB form hidden inputs use 0-based index and only increment formCount *after* this function is run.
+                Therefore formcount and order are currently equal and order must be incremented
+                to ensure it's *greater* than previous item */
+                $(fixPrefix('#id_' + opts.emptyChildFormPrefix + '-ORDER')).val(formCount + 1);
             }
             self.updateMoveButtonDisabledStates();
 
@@ -320,8 +329,6 @@ function initCollapsibleBlocks(){
 }
 
 $(function() {
-    initDateChoosers();
-    initTimeChoosers();
     initSlugAutoPopulate();
     initSlugCleaning();
     initErrorDetection();
@@ -332,36 +339,67 @@ $(function() {
     });
 
     /* Set up behaviour of preview button */
-    $('.action-preview').click(function() {
-        var previewWindow = window.open($(this).data('placeholder'), $(this).data('windowname'));
+    $('.action-preview').click(function(e) {
+        e.preventDefault();
+        var $this = $(this);
 
-        $.ajax({
-            type: "POST",
-            url: $(this).data('action'),
-            data: $('#page-edit-form').serialize(),
-            success: function(data, textStatus, request) {
-                if (request.getResponseHeader('X-Wagtail-Preview') == 'ok') {
-                    previewWindow.document.open();
-                    previewWindow.document.write(data);
-                    previewWindow.document.close();
-                } else {
-                    previewWindow.close();
-                    document.open();
-                    document.write(data);
-                    document.close();
-                }
-            },
-            error: function(xhr, textStatus, errorThrown) {
-                /* If an error occurs, display it in the preview window so that
-                we aren't just showing the spinner forever. We preserve the original
-                error output rather than giving a 'friendly' error message so that
-                developers can debug template errors. (On a production site, we'd
-                typically be serving a friendly custom 500 page anyhow.) */
-                previewWindow.document.open();
-                previewWindow.document.write(xhr.responseText);
-                previewWindow.document.close();
+        var previewWindow = window.open($this.data('placeholder'), $this.data('windowname'));
+        
+        if(/MSIE/.test(navigator.userAgent)){
+            submitPreview.call($this, false);
+        } else {
+            previewWindow.onload = function(){
+                submitPreview.call($this, true);
             }
-        });
-        return false;
+        }
+
+        function submitPreview(enhanced){
+            $.ajax({
+                type: "POST",
+                url: $this.data('action'),
+                data: $('#page-edit-form').serialize(),
+                success: function(data, textStatus, request) {
+                    if (request.getResponseHeader('X-Wagtail-Preview') == 'ok') {
+                        var pdoc = previewWindow.document;
+                        
+                        if(enhanced){
+                            var frame = pdoc.getElementById('preview-frame');
+
+                            frame = frame.contentWindow || frame.contentDocument.document || frame.contentDocument;
+                            frame.document.open();
+                            frame.document.write(data);                 
+                            frame.document.close();
+
+                            var hideTimeout = setTimeout(function(){
+                                pdoc.getElementById('loading-spinner-wrapper').className += 'remove';
+                                clearTimeout(hideTimeout);
+                            }) // just enough to give effect without adding discernible slowness                       
+                        } else {
+                            pdoc.open();
+                            pdoc.write(data);                 
+                            pdoc.close()
+                        }
+                    } else {
+                        previewWindow.close();
+                        document.open();
+                        document.write(data);
+                        document.close();
+                    }
+                },
+                error: function(xhr, textStatus, errorThrown) {
+                    /* If an error occurs, display it in the preview window so that
+                    we aren't just showing the spinner forever. We preserve the original
+                    error output rather than giving a 'friendly' error message so that
+                    developers can debug template errors. (On a production site, we'd
+                    typically be serving a friendly custom 500 page anyhow.) */
+
+                    previewWindow.document.open();
+                    previewWindow.document.write(xhr.responseText);
+                    previewWindow.document.close();
+                }
+            });
+
+        }
+        
     });
 });
