@@ -1,3 +1,5 @@
+import warnings
+
 from django.http import Http404, HttpResponse
 from django.shortcuts import render, redirect, get_object_or_404
 from django.core.exceptions import ValidationError, PermissionDenied
@@ -5,14 +7,18 @@ from django.contrib import messages
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.auth.decorators import permission_required
 from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
+from django.utils import timezone
 from django.utils.translation import ugettext as _
+from django.views.decorators.http import require_GET
 from django.views.decorators.vary import vary_on_headers
 
 from wagtail.wagtailadmin.edit_handlers import TabbedInterface, ObjectList
 from wagtail.wagtailadmin.forms import SearchForm
-from wagtail.wagtailadmin import tasks, hooks
+from wagtail.wagtailadmin import tasks, signals
 
-from wagtail.wagtailcore.models import Page, PageRevision, get_page_types
+from wagtail.wagtailcore import hooks
+from wagtail.wagtailcore.models import Page, PageRevision
+from wagtail.wagtailcore.signals import page_published
 
 
 @permission_required('wagtailadmin.access_admin')
@@ -25,14 +31,23 @@ def index(request, parent_page_id=None):
     pages = parent_page.get_children().prefetch_related('content_type')
 
     # Get page ordering
-    if 'ordering' in request.GET:
-        ordering = request.GET['ordering']
-
-        if ordering in ['title', '-title', 'content_type', '-content_type', 'live', '-live']:
-            pages = pages.order_by(ordering)
-    else:
+    ordering = request.GET.get('ordering', 'title')
+    if ordering not in ['title', '-title', 'content_type', '-content_type', 'live', '-live', 'ord']:
         ordering = 'title'
 
+    # Pagination
+    if ordering != 'ord':
+        pages = pages.order_by(ordering)
+
+        p = request.GET.get('p', 1)
+        paginator = Paginator(pages, 50)
+        try:
+            pages = paginator.page(p)
+        except PageNotAnInteger:
+            pages = paginator.page(1)
+        except EmptyPage:
+            pages = paginator.page(paginator.num_pages)
+
     return render(request, 'wagtailadmin/pages/index.html', {
         'parent_page': parent_page,
         'ordering': ordering,
@@ -46,13 +61,17 @@ def add_subpage(request, parent_page_id):
     if not parent_page.permissions_for_user(request.user).can_add_subpage():
         raise PermissionDenied
 
-    page_types = sorted([ContentType.objects.get_for_model(model_class) for model_class in parent_page.clean_subpage_types()], key=lambda pagetype: pagetype.name.lower())
-    all_page_types = sorted(get_page_types(), key=lambda pagetype: pagetype.name.lower())
+    page_types = sorted(parent_page.clean_subpage_types(), key=lambda pagetype: pagetype.name.lower())
+
+    if len(page_types) == 1:
+        # Only one page type is available - redirect straight to the create form rather than
+        # making the user choose
+        content_type = page_types[0]
+        return redirect('wagtailadmin_pages_create', content_type.app_label, content_type.model, parent_page.id)
 
     return render(request, 'wagtailadmin/pages/add_subpage.html', {
         'parent_page': parent_page,
         'page_types': page_types,
-        'all_page_types': all_page_types,
     })
 
 
@@ -102,15 +121,16 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
     except ContentType.DoesNotExist:
         raise Http404
 
+    # Get class
     page_class = content_type.model_class()
 
+    # Make sure the class is a descendant of Page
+    if not issubclass(page_class, Page):
+        raise Http404
+
     # page must be in the list of allowed subpage types for this parent ID
-    # == Restriction temporarily relaxed so that as superusers we can add index pages and things -
-    # == TODO: reinstate this for regular editors when we have distinct user types
-    #
-    # if page_class not in parent_page.clean_subpage_types():
-    #     messages.error(request, "Sorry, you do not have access to create a page of type '%s' here." % content_type.name)
-    #     return redirect('wagtailadmin_pages_select_type')
+    if content_type not in parent_page.clean_subpage_types():
+        raise PermissionDenied
 
     page = page_class(owner=request.user)
     edit_handler_class = get_page_edit_handler(page_class)
@@ -127,23 +147,66 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
             return slug
         form.fields['slug'].clean = clean_slug
 
+        # Stick another validator into the form to check that the scheduled publishing settings are set correctly
+        def clean():
+            cleaned_data = form_class.clean(form)
+
+            # Go live must be before expire
+            go_live_at = cleaned_data.get('go_live_at')
+            expire_at = cleaned_data.get('expire_at')
+
+            if go_live_at and expire_at:
+                if go_live_at > expire_at:
+                    msg = _('Go live date/time must be before expiry date/time')
+                    form._errors['go_live_at'] = form.error_class([msg])
+                    form._errors['expire_at'] = form.error_class([msg])
+                    del cleaned_data['go_live_at']
+                    del cleaned_data['expire_at']
+
+            # Expire must be in the future
+            expire_at = cleaned_data.get('expire_at')
+
+            if expire_at and expire_at < timezone.now():
+                form._errors['expire_at'] = form.error_class([_('Expiry date/time must be in the future')])
+                del cleaned_data['expire_at']
+
+            return cleaned_data
+        form.clean = clean
+
         if form.is_valid():
             page = form.save(commit=False)  # don't save yet, as we need treebeard to assign tree params
 
             is_publishing = bool(request.POST.get('action-publish')) and parent_page_perms.can_publish_subpage()
             is_submitting = bool(request.POST.get('action-submit'))
+            go_live_at = form.cleaned_data.get('go_live_at')
+            future_go_live = go_live_at and go_live_at > timezone.now()
+            approved_go_live_at = None
 
             if is_publishing:
-                page.live = True
                 page.has_unpublished_changes = False
+                page.expired = False
+                if future_go_live:
+                    page.live = False
+                    # Set approved_go_live_at only if is publishing
+                    # and the future_go_live is actually in future
+                    approved_go_live_at = go_live_at
+                else:
+                    page.live = True
             else:
                 page.live = False
                 page.has_unpublished_changes = True
 
             parent_page.add_child(instance=page)  # assign tree parameters - will cause page to be saved
-            page.save_revision(user=request.user, submitted_for_moderation=is_submitting)
+
+            # Pass approved_go_live_at to save_revision
+            page.save_revision(
+                user=request.user,
+                submitted_for_moderation=is_submitting,
+                approved_go_live_at=approved_go_live_at
+            )
 
             if is_publishing:
+                page_published.send(sender=page_class, instance=page)
                 messages.success(request, _("Page '{0}' published.").format(page.title))
             elif is_submitting:
                 messages.success(request, _("Page '{0}' submitted for moderation.").format(page.title))
@@ -158,9 +221,10 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
 
             return redirect('wagtailadmin_explore', page.get_parent().id)
         else:
-            messages.error(request, _("The page could not be created due to errors."))
+            messages.error(request, _("The page could not be created due to validation errors"))
             edit_handler = edit_handler_class(instance=page, form=form)
     else:
+        signals.init_new_page.send(sender=create, page=page, parent=parent_page)
         form = form_class(instance=page)
         edit_handler = edit_handler_class(instance=page, form=form)
 
@@ -169,7 +233,8 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
         'page_class': page_class,
         'parent_page': parent_page,
         'edit_handler': edit_handler,
-        'display_modes': page.get_page_modes(),
+        'preview_modes': page.preview_modes,
+        'form': form, # Used in unit tests
     })
 
 
@@ -199,15 +264,54 @@ def edit(request, page_id):
             return slug
         form.fields['slug'].clean = clean_slug
 
+        # Stick another validator into the form to check that the scheduled publishing settings are set correctly
+        def clean():
+            cleaned_data = form_class.clean(form)
+
+            # Go live must be before expire
+            go_live_at = cleaned_data.get('go_live_at')
+            expire_at = cleaned_data.get('expire_at')
+
+            if go_live_at and expire_at:
+                if go_live_at > expire_at:
+                    msg = _('Go live date/time must be before expiry date/time')
+                    form._errors['go_live_at'] = form.error_class([msg])
+                    form._errors['expire_at'] = form.error_class([msg])
+                    del cleaned_data['go_live_at']
+                    del cleaned_data['expire_at']
+
+            # Expire must be in the future
+            expire_at = cleaned_data.get('expire_at')
+
+            if expire_at and expire_at < timezone.now():
+                form._errors['expire_at'] = form.error_class([_('Expiry date/time must be in the future')])
+                del cleaned_data['expire_at']
+
+            return cleaned_data
+        form.clean = clean
+
         if form.is_valid():
             is_publishing = bool(request.POST.get('action-publish')) and page_perms.can_publish()
             is_submitting = bool(request.POST.get('action-submit'))
+            go_live_at = form.cleaned_data.get('go_live_at')
+            future_go_live = go_live_at and go_live_at > timezone.now()
+            approved_go_live_at = None
 
             if is_publishing:
-                page.live = True
                 page.has_unpublished_changes = False
+                page.expired = False
+                if future_go_live:
+                    page.live = False
+                    # Set approved_go_live_at only if publishing
+                    approved_go_live_at = go_live_at
+                else:
+                    page.live = True
                 form.save()
-                page.revisions.update(submitted_for_moderation=False)
+                # Clear approved_go_live_at for older revisions
+                page.revisions.update(
+                    submitted_for_moderation=False,
+                    approved_go_live_at=None,
+                )
             else:
                 # not publishing the page
                 if page.live:
@@ -219,9 +323,14 @@ def edit(request, page_id):
                     page.has_unpublished_changes = True
                     form.save()
 
-            page.save_revision(user=request.user, submitted_for_moderation=is_submitting)
+            page.save_revision(
+                user=request.user,
+                submitted_for_moderation=is_submitting,
+                approved_go_live_at=approved_go_live_at
+            )
 
             if is_publishing:
+                page_published.send(sender=page.__class__, instance=page)
                 messages.success(request, _("Page '{0}' published.").format(page.title))
             elif is_submitting:
                 messages.success(request, _("Page '{0}' submitted for moderation.").format(page.title))
@@ -237,10 +346,11 @@ def edit(request, page_id):
             return redirect('wagtailadmin_explore', page.get_parent().id)
         else:
             messages.error(request, _("The page could not be saved due to validation errors"))
+
             edit_handler = edit_handler_class(instance=page, form=form)
             errors_debug = (
                 repr(edit_handler.form.errors)
-                + repr([(name, formset.errors) for (name, formset) in edit_handler.form.formsets.iteritems() if formset.errors])
+                + repr([(name, formset.errors) for (name, formset) in edit_handler.form.formsets.items() if formset.errors])
             )
     else:
         form = form_class(instance=page)
@@ -254,7 +364,8 @@ def edit(request, page_id):
         'page': page,
         'edit_handler': edit_handler,
         'errors_debug': errors_debug,
-        'display_modes': page.get_page_modes(),
+        'preview_modes': page.preview_modes,
+        'form': form, # Used in unit tests
     })
 
 
@@ -285,7 +396,28 @@ def delete(request, page_id):
 @permission_required('wagtailadmin.access_admin')
 def view_draft(request, page_id):
     page = get_object_or_404(Page, id=page_id).get_latest_revision_as_page()
-    return page.serve(request)
+    return page.serve_preview(page.dummy_request(), page.default_preview_mode)
+
+
+def get_preview_response(page, preview_mode):
+    """
+    Helper function for preview_on_edit and preview_on_create -
+    return a page's preview response via either serve_preview or the deprecated
+    show_as_mode method
+    """
+    # Check the deprecated Page.show_as_mode method, as subclasses of Page
+    # might be overriding that to return a response
+    response = page.show_as_mode(preview_mode)
+    if response:
+        warnings.warn(
+            "Defining 'show_as_mode' on a page model is deprecated. Use 'serve_preview' instead",
+            DeprecationWarning
+        )
+        return response
+    else:
+        # show_as_mode did not return a response, so go ahead and use the 'proper'
+        # serve_preview method
+        return page.serve_preview(page.dummy_request(), preview_mode)
 
 
 @permission_required('wagtailadmin.access_admin')
@@ -301,19 +433,8 @@ def preview_on_edit(request, page_id):
     if form.is_valid():
         form.save(commit=False)
 
-        # This view will generally be invoked as an AJAX request; as such, in the case of
-        # an error Django will return a plaintext response. This isn't what we want, since
-        # we will be writing the response back to an HTML page regardless of success or
-        # failure - as such, we strip out the X-Requested-With header to get Django to return
-        # an HTML error response
-        request.META.pop('HTTP_X_REQUESTED_WITH', None)
-
-        try:
-            display_mode = request.GET['mode']
-        except KeyError:
-            display_mode = page.get_page_modes()[0][0]
-
-        response = page.show_as_mode(display_mode)
+        preview_mode = request.GET.get('mode', page.default_preview_mode)
+        response = get_preview_response(page, preview_mode)
 
         response['X-Wagtail-Preview'] = 'ok'
         return response
@@ -324,7 +445,7 @@ def preview_on_edit(request, page_id):
         response = render(request, 'wagtailadmin/pages/edit.html', {
             'page': page,
             'edit_handler': edit_handler,
-            'display_modes': page.get_page_modes(),
+            'preview_modes': page.preview_modes,
         })
         response['X-Wagtail-Preview'] = 'error'
         return response
@@ -353,18 +474,12 @@ def preview_on_create(request, content_type_app_name, content_type_model_name, p
         parent_page = get_object_or_404(Page, id=parent_page_id).specific
         page.set_url_path(parent_page)
 
-        # This view will generally be invoked as an AJAX request; as such, in the case of
-        # an error Django will return a plaintext response. This isn't what we want, since
-        # we will be writing the response back to an HTML page regardless of success or
-        # failure - as such, we strip out the X-Requested-With header to get Django to return
-        # an HTML error response
-        request.META.pop('HTTP_X_REQUESTED_WITH', None)
+        # Set treebeard attributes
+        page.depth = parent_page.depth + 1
+        page.path = Page._get_children_path_interval(parent_page.path)[1]
 
-        try:
-            display_mode = request.GET['mode']
-        except KeyError:
-            display_mode = page.get_page_modes()[0][0]
-        response = page.show_as_mode(display_mode)
+        preview_mode = request.GET.get('mode', page.default_preview_mode)
+        response = get_preview_response(page, preview_mode)
 
         response['X-Wagtail-Preview'] = 'ok'
         return response
@@ -378,13 +493,13 @@ def preview_on_create(request, content_type_app_name, content_type_model_name, p
             'page_class': page_class,
             'parent_page': parent_page,
             'edit_handler': edit_handler,
-            'display_modes': page.get_page_modes(),
+            'preview_modes': page.preview_modes,
         })
         response['X-Wagtail-Preview'] = 'error'
         return response
 
 
-def preview_placeholder(request):
+def preview(request):
     """
     The HTML of a previewed page is written to the destination browser window using document.write.
     This overwrites any previous content in the window, while keeping its URL intact. This in turn
@@ -405,8 +520,13 @@ def preview_placeholder(request):
     Since we're going to this trouble, we'll also take the opportunity to display a spinner on the
     placeholder page, providing some much-needed visual feedback.
     """
-    return render(request, 'wagtailadmin/pages/preview_placeholder.html')
+    return render(request, 'wagtailadmin/pages/preview.html')
 
+def preview_loading(request):
+    """
+    This page is blank, but must be real HTML so its DOM can be written to once the preview of the page has rendered
+    """
+    return HttpResponse("<html><head><title></title></head><body></body></html>")
 
 @permission_required('wagtailadmin.access_admin')
 def unpublish(request, page_id):
@@ -418,6 +538,8 @@ def unpublish(request, page_id):
         parent_id = page.get_parent().id
         page.live = False
         page.save()
+        # Since page is unpublished clear the approved_go_live_at of all revisions
+        page.revisions.update(approved_go_live_at=None)
         messages.success(request, _("Page '{0}' unpublished.").format(page.title))
         return redirect('wagtailadmin_explore', parent_id)
 
@@ -520,7 +642,8 @@ def get_page_edit_handler(page_class):
     if page_class not in PAGE_EDIT_HANDLERS:
         PAGE_EDIT_HANDLERS[page_class] = TabbedInterface([
             ObjectList(page_class.content_panels, heading='Content'),
-            ObjectList(page_class.promote_panels, heading='Promote')
+            ObjectList(page_class.promote_panels, heading='Promote'),
+            ObjectList(page_class.settings_panels, heading='Settings', classname="settings")
         ])
 
     return PAGE_EDIT_HANDLERS[page_class]
@@ -580,6 +703,7 @@ def approve_moderation(request, revision_id):
 
     if request.POST:
         revision.publish()
+        page_published.send(sender=revision.page.__class__, instance=revision.page.specific)
         messages.success(request, _("Page '{0}' published.").format(revision.page.title))
         tasks.send_notification.delay(revision.id, 'approved', request.user.id)
 
@@ -606,6 +730,7 @@ def reject_moderation(request, revision_id):
 
 
 @permission_required('wagtailadmin.access_admin')
+@require_GET
 def preview_for_moderation(request, revision_id):
     revision = get_object_or_404(PageRevision, id=revision_id)
     if not revision.page.permissions_for_user(request.user).can_publish():
@@ -619,4 +744,6 @@ def preview_for_moderation(request, revision_id):
 
     request.revision_id = revision_id
 
-    return page.serve(request)
+    # pass in the real user request rather than page.dummy_request(), so that request.user
+    # and request.revision_id will be picked up by the wagtail user bar
+    return page.serve_preview(request, page.default_preview_mode)
