@@ -2,8 +2,9 @@
 A generic HTML whitelisting engine, designed to accommodate subclassing to override
 specific rules.
 """
+from six.moves.urllib.parse import urlparse
+
 from bs4 import BeautifulSoup, NavigableString, Tag
-from urlparse import urlparse
 
 
 ALLOWED_URL_SCHEMES = ['', 'http', 'https', 'ftp', 'mailto', 'tel']
@@ -28,7 +29,7 @@ def attribute_rule(allowed_attrs):
     * if the lookup returns a truthy value, keep the attribute; if falsy, drop it
     """
     def fn(tag):
-        for attr, val in tag.attrs.items():
+        for attr, val in list(tag.attrs.items()):
             rule = allowed_attrs.get(attr)
             if rule:
                 if callable(rule):
@@ -65,7 +66,8 @@ class Whitelister(object):
         'h6': allow_without_attributes,
         'hr': allow_without_attributes,
         'i': allow_without_attributes,
-        'img': attribute_rule({'src': check_url, 'width': True, 'height': True, 'alt': True}),
+        'img': attribute_rule({'src': check_url, 'width': True, 'height': True,
+                               'alt': True}),
         'li': allow_without_attributes,
         'ol': allow_without_attributes,
         'p': allow_without_attributes,
@@ -77,10 +79,11 @@ class Whitelister(object):
 
     @classmethod
     def clean(cls, html):
-        """Clean up an HTML string to contain just the allowed elements / attributes"""
+        """Clean up an HTML string to contain just the allowed elements /
+        attributes"""
         doc = BeautifulSoup(html, 'lxml')
         cls.clean_node(doc, doc)
-        return unicode(doc)
+        return doc.decode()
 
     @classmethod
     def clean_node(cls, doc, node):
@@ -89,7 +92,10 @@ class Whitelister(object):
             cls.clean_string_node(doc, node)
         elif isinstance(node, Tag):
             cls.clean_tag_node(doc, node)
-        else:
+        # This branch is here in case node is a BeautifulSoup object that does
+        # not inherit from NavigableString or Tag. I can't find any examples
+        # of such a thing at the moment, so this branch is untested.
+        else:  # pragma: no cover
             cls.clean_unknown_node(doc, node)
 
     @classmethod
