@@ -1,6 +1,5 @@
 from __future__ import division  # Use true division
 
-from django.utils.html import escape
 from django.template.loader import render_to_string
 
 from wagtail.wagtailembeds import get_embed
@@ -17,7 +16,7 @@ def embed_to_frontend_html(url):
                 ratio = "0"
 
             # Render template
-            render_to_string('wagtailembeds/embed_frontend.html', {
+            return render_to_string('wagtailembeds/embed_frontend.html', {
                 'embed': embed,
                 'ratio': ratio,
             })
