@@ -1,6 +1,9 @@
-from base import BaseImageBackend
+from __future__ import absolute_import
+
+from .base import BaseImageBackend
 import PIL.Image
 
+
 class PillowBackend(BaseImageBackend):
     def __init__(self, params):
         super(PillowBackend, self).__init__(params)
@@ -32,4 +35,4 @@ class PillowBackend(BaseImageBackend):
         top = (original_height - final_height) / 2
         return image.crop(
             (left, top, left + final_width, top + final_height)
-        )
\ No newline at end of file
+        )
